package com.fjhx.service.material.impl;

import cn.hutool.core.convert.Convert;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.entity.classify.Classify;
import com.fjhx.entity.material.Material;
import com.fjhx.entity.stock.Stock;
import com.fjhx.mapper.material.MaterialMapper;
import com.fjhx.params.material.MaterialVo;
import com.fjhx.service.classify.ClassifyService;
import com.fjhx.service.material.MaterialService;
import com.fjhx.service.stock.StockService;
import com.fjhx.uitl.code.CodeEnum;
import com.fjhx.utils.wrapperUtil.IWrapper;
import org.springblade.core.log.exception.ServiceException;
import org.springblade.core.redis.lock.RedisLockClient;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.system.entity.Dept;
import org.springblade.system.feign.ISysClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 物料 服务实现类
 * </p>
 *
 * @author ${author}
 * @since 2022-11-24
 */
@Service
public class MaterialServiceImpl extends ServiceImpl<MaterialMapper, Material> implements MaterialService {

    @Lazy
    @Autowired
    private ClassifyService classifyService;

    @Autowired
    private StockService stockService;

    @Autowired
    private RedisLockClient redisLockClient;

    @Autowired
    private ISysClient iSysClient;

    private final String REDIS_LOCK_CACHE_KEY = "seq:lock:" + AuthUtil.getTenantId() + ":material:";

    @Override
    public Page<Material> getPage(Map<String, Object> condition) {

        IWrapper<Material> wrapper = IWrapper.getWrapper(condition);

        //处理分类
        List<Long> classifyIds = new ArrayList<>();
        if (Func.isNotEmpty(condition.get("classifyId"))) {
            classifyIds.add(Convert.toLong(condition.get("classifyId")));
            //查询所有子级
            List<Classify> classifies = classifyService.lambdaQuery().apply("FIND_IN_SET(" + condition.get("classifyId") + ", parent_id_set)").list();
            if (Func.isNotEmpty(classifies)) {
                classifyIds.addAll(classifies.stream().map(Classify::getId).distinct().collect(Collectors.toList()));
            }
        }

        //处理部门
        List<Long> deptIds = new ArrayList<>();
        if (Func.isNotEmpty(condition.get("deptId"))) {
            deptIds.add(Convert.toLong(condition.get("deptId")));
            //查询所有子级
            List<Dept> depts = iSysClient.getDeptAncestorsById(Convert.toLong(condition.get("deptId")));
            if (Func.isNotEmpty(depts)) {
                deptIds.addAll(depts.stream().map(Dept::getId).distinct().collect(Collectors.toList()));
            }
        }

        wrapper.eq(Material::getType)
                .in(Material::getClassifyId, classifyIds)
                .in(Material::getDeptId, deptIds)
                .apply(Func.isNotEmpty(condition.get("code")), "instr(`code`, '" + condition.get("code") + "') > 0")
                .apply(Func.isNotEmpty(condition.get("name")), "instr(`name`, '" + condition.get("name") + "') > 0")
                .and(Func.isNotEmpty(condition.get("keyword")), o -> o.apply("instr(`code`, '" + condition.get("keyword") + "') > 0").or().apply("instr(`name`, '" + condition.get("keyword") + "') > 0"))
                .orderByDesc(Material::getId);

        Page<Material> page = page(condition, wrapper);
        if (Func.isNotEmpty(page.getRecords())) {
            deptIds = page.getRecords().stream().map(Material::getDeptId).distinct().collect(Collectors.toList());
            //查询部门
            Map<Long, Dept> deptMap = iSysClient.getDeptByIdsToMap(deptIds);

            Map<Long, Stock> stockMap = new HashMap<>();
            //查询库存量
            if (Func.isNotEmpty(condition.get("warehouseId"))) {
                //物料ID集合
                List<Long> ids = page.getRecords().stream().map(Material::getId).distinct().collect(Collectors.toList());
                List<Stock> stocks = stockService.lambdaQuery().eq(Stock::getWarehouseId, condition.get("warehouseId")).in(Stock::getGoodsId, ids).list();
                if (Func.isNotEmpty(stocks)) {
                    stockMap = stocks.stream().collect(Collectors.toMap(Stock::getGoodsId, Function.identity(), (key1, key2) -> key2));
                }
            }

            for (Material record : page.getRecords()) {
                if (Func.isNotEmpty(deptMap) && Func.isNotEmpty(deptMap.get(record.getDeptId()))) {
                    Dept dept = deptMap.get(record.getDeptId());
                    record.setDeptName(dept.getDeptName());
                }

                if (Func.isNotEmpty(stockMap) && Func.isNotEmpty(stockMap.get(record.getId()))) {
                    Stock stock = stockMap.get(record.getId());
                    record.setStockQuantity(stock.getQuantity());
                }
            }
        }

        return page;
    }

    @Override
    public void add(MaterialVo materialVo) {
        //查询名称是否已存在
        if (!checkNameIsExist(null, materialVo.getName())) {
            throw new ServiceException("该物料名称已存在：" + materialVo.getName());
        }

        if (
                !redisLockClient.lockFair(REDIS_LOCK_CACHE_KEY, () -> {
                    //处理编码
                    materialVo.setCode(CodeEnum.MATERIAL.getCode(materialVo.getCode()));
                    save(materialVo);
                    return true;
                })
        ) {
            throw new ServiceException("当前系统繁忙，请稍后重试！");
        }
    }

    private Boolean checkNameIsExist(Long id, String name) {
        List<Material> list = lambdaQuery().ne(Func.isNotEmpty(id), Material::getId, id).eq(Material::getName, name).list();
        return !Func.isNotEmpty(list);
    }

    @Override
    public void edit(MaterialVo materialVo) {
        //查询名称是否已存在
        if (!checkNameIsExist(materialVo.getId(), materialVo.getName())) {
            throw new ServiceException("该物料名称已存在：" + materialVo.getName());
        }
        updateById(materialVo);
    }

    @Override
    public void delete(MaterialVo materialVo) {
        //查询是否存在库存
        List<Stock> stocks = stockService.lambdaQuery().eq(Stock::getGoodsId, materialVo.getId()).list();
        if (Func.isNotEmpty(stocks)) {
            throw new ServiceException("该物料还有库存，无法删除！");
        }
        removeById(materialVo.getId());
    }

}
